------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004 - 2008, National Optical Astronomy Observatory, 
-- which is operated for the National Science Foundation by the    
-- Association of Universities for Research in Astronomy under    
-- Cooperative Agreement No. AST-0132798.                                                                                                                           
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   TOP_LEVEL  
-- MODULE       :   REGISTER CONTROL
-- VERSION      :   2.20
------------------------------------------------------------------------
-- FILE NAME    :   Dummy_RegisterControl_Ver220.vhd
-- AUTHOR         :   Peter Moore
-- DATE               :   03/05/2012
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  :  Use this module to provide a skeleton wish bone module          
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	     AUTHOR  VERSION  MODIFICATIONS
-- 09/12/2009  PCM     1.00    INITIAL VERSION
-- 01/31/2011    pcm      2.03    Dusted off and pulled into service for the spare module addresses in Rev 2.03 firmware - this since Nick complains that addressing a non-existant thing returns nothing .. like it isn't there :-)
-- 02/07/2011  pcm       2.04    Added dummy I/O port to test hypothesis concerning dropped I/O pins via the Xilinx tool.
-- 03/05/2012   PCM     2.20   Version value only .. This allows a firmware recovery point to be set at system version 2.11 
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- Wishbone Interface:
-- WbSysClk             sub-module master clock (input)
-- WbReset              sub-module master reset signal (input)
-- WbWriteEnable        Write strobe for sub-modules (input from WB slave)
-- WbAddress            16-bit sub-module address valid for write / read transactions (input)
-- WbDataIn             32-bit data input from WB slave for Wishbone read transactions
-- WbWrtAck             Write acknowledge to WB slave for write transaction (output)
-- WbErrorAck           Error marker to WB slave for write or read transaction (output)
-- WbBusy               Busy / holdoff strobe to WB slave during write / read transaction (output)
-- WbDataOut            32-bit data to WB slave valid during write transactions (output)
--
-- LCB Control Parameters
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Dummy_RegisterControl_Ver220 is
	Port ( 
-- WishBone bus signals
		WbSysClk 				: in   std_logic;
		WbReset 					: in   std_logic;
		WbWriteEnable 			: in   std_logic;
		WbAddress 				: in   std_logic_vector(15 downto 0);
		WbDataIn 				: in   std_logic_vector(31 downto 0);
		WbWrtAck 				: out  std_logic;
		WbErrAck 				: out  std_logic;
		WbBusy 					: out  std_logic;
		WbDataOut 				: out  std_logic_vector(31 downto 0);
      WbStatOut            : out  std_logic_vector(7 downto 0);
      WbStatIn             : in   std_logic_vector(31 downto 0);
-- I/O section
      DummyIo0_o           : out  std_logic;
      DummyIo1_o           : out  std_logic;
      DummyIo2_o           : out  std_logic;
      
-- Local register data
      StartExpVector_i     : in   std_logic_vector(15 downto 0);
      StartExpStb_i        : in   std_logic
-- Local status data
	);

end Dummy_RegisterControl_Ver220;

architecture Behavioral of Dummy_RegisterControl_Ver220 is

-- Firmware module code revision value
   constant CODE_IDENT   : std_logic_vector(15 downto 0) := x"00DC"; -- Current firmware version is 2.20
   constant LCB_CONTROL_MODULE_IDENT : std_logic_vector(15 downto 0) := x"03E7"; -- Dummy Module identity code is 999   
-- Misc bits --
   signal AddrErr : std_logic;      -- Indicates an addressing mismatch in this module i.e. bus master is trying to write to a non existant register
	signal wrt_ack, err_ack : std_logic;

-- Decoding Vectors
   signal CS16K_0, CS16K_3  : std_logic; -- CS16K_1, CS16K_2 
   signal CS4K_0, CS4K_3    : std_logic; -- CS4K_1, CS4K_2, 
   signal CS1K_0, CS1K_3    : std_logic; -- CS1K_1, CS1K_2, 
   signal CS256_0, CS256_1, CS256_3 : std_logic; -- CS256_2, 
   signal CS64_0, CS64_3    : std_logic; -- CS64_1, CS64_2, 
   signal CS16_0, CS16_3    : std_logic; -- CS16_1, CS16_2, 
   signal CS_0 : std_logic; -- , CS_1, CS_2, CS_3, CS_4, CS_5, CS_6, CS_7
   signal CS_C, CS_D, CS_E, CS_F : std_logic;  -- CS_8, CS_9, CS_A, CS_B, 
    
-- Misc registers and selects
   signal CodeId_cs, ModuleId_cs : std_logic;
   signal DummyCtrlModInStatus_cs, DummyCtrlModOutStatus_cs : std_logic;
	signal Address : std_logic_vector(15 downto 0);
	signal Data_in : std_logic_vector(31 downto 0);
   signal LocalStatus_reg : std_logic_vector(31 downto 0);   
   signal StartExp_cs : std_logic;
   signal StartExp_reg : std_logic_vector(15 downto 0);
   signal StatOut_cs   : std_logic;
   signal StatOut_reg  : std_logic_vector(31 downto 0);
   
-- Signals for the Application Interface module
   
begin
------------------------------------------------------------------------
-- Combinational and Static logic assignments
------------------------------------------------------------------------
	Address <= WbAddress;
	Data_in <= WbDataIn;
	WbBusy <= '0';									-- Not used
	WbWrtAck <= wrt_ack;
	WbErrAck <= err_ack;
   LocalStatus_reg <= WbStatIn;  

-- Status input bits
   
-- Status output bits
   WbStatOut  <= StatOut_reg(7 downto 0);
   
-- I/O outputs
  DummyIo0_o  <= StatOut_reg(0);
  DummyIo1_o  <= StatOut_reg(1);
  DummyIo2_o  <= StatOut_reg(2);
  
------------------------------------------------------------------------
-- MODULE ADDRESS DECODE LOGIC. Enable the decoded address strobes as required.
------------------------------------------------------------------------	
-- Decode 16K word Blocks
    CS16K_0 <= (not Address(15)) and (not Address(14));   -- 0x0000 => 0x3FFF
--    CS16K_1 <= (not Address(15)) and Address(14);         -- 0x4000 => 0x7FFF
--    CS16K_2 <= Address(15) and (not Address(14));         -- 0x8000 => 0xBFFF
    CS16K_3 <= Address(15) and Address(14);               -- 0xC000 => 0xFFFF

-- Decode 4K word Blocks
    CS4K_0 <= (not Address(13)) and (not Address(12));    --0x0000 => 0x0FFF, 0x4000 => 0x4FFF, 0x8000 => 0x8FFF, 0xC000 => 0xCFFF
--    CS4K_1 <= (not Address(13)) and Address(12);          --0x1000 => 0x1FFF, 0x5000 => 0x5FFF, 0x9000 => 0x9FFF, 0xD000 => 0xDFFF
--    CS4K_2 <= Address(13) and (not Address(12));          --0x2000 => 0x2FFF, 0x6000 => 0x6FFF, 0xA000 => 0xAFFF, 0xE000 => 0xEFFF
    CS4K_3 <= Address(13) and Address(12);                -- 0x3000 => 0x3FFF, 0x7000 => 0x7FFF, 0xB000 => 0xBFFF, 0xF000 => 0xFFFF
     
-- Decode 1K word Blocks
    CS1K_0 <= (not Address(11)) and (not Address(10));   -- 0x_000 => 0x_3FF
--    CS1K_1 <= (not Address(11)) and Address(10);         -- 0x_400 => 0x_7FF
--    CS1K_2 <= Address(11) and (not Address(10));         -- 0x_800 => 0x_BFF
    CS1K_3 <= Address(11) and Address(10);               -- 0x_C00 => 0x_FFF

-- Decode 256 word Blocks
    CS256_0 <= (not Address(9)) and (not Address(8));    --0x_000 => 0x_0FF, 0x_400 => 0x_4FF, 0x_800 => 0x_8FF, 0x_C00 => 0x_CFF
    CS256_1 <= (not Address(9)) and Address(8);          --0x_100 => 0x_1FF, 0x_500 => 0x_5FF, 0x_900 => 0x_9FF, 0x_D00 => 0x_DFF
--    CS256_2 <= Address(9) and (not Address(8));          --0x_200 => 0x_2FF, 0x_600 => 0x_6FF, 0x_A00 => 0x_AFF, 0x_E00 => 0x_EFF
    CS256_3 <= Address(9) and Address(8);                -- 0x_300 => 0x_3FF, 0x_700 => 0x_7FF, 0x_B00 => 0x_BFF, 0x_F00 => 0x_FFF

-- Decode 64 word Blocks
    CS64_0 <= (not Address(7)) and (not Address(6));   -- 0x__00 => 0x__3F
--    CS64_1 <= (not Address(7)) and Address(6);         -- 0x__40 => 0x__7F
--    CS64_2 <= Address(7) and (not Address(6));         -- 0x__80 => 0x__BF
    CS64_3 <= Address(7) and Address(6);               -- 0x__C0 => 0x__FF

 -- Decode 16 word Blocks
    CS16_0 <= (not Address(5)) and (not Address(4));    --0x__00 => 0x__0F, 0x__40 => 0x__4F, 0x__80 => 0x__8F, 0x__C0 => 0x__CF
--    CS16_1 <= (not Address(5)) and Address(4);          --0x__10 => 0x__1F, 0x__50 => 0x__5F, 0x__90 => 0x__9F, 0x__D0 => 0x__DF
--    CS16_2 <= Address(5) and (not Address(4));          --0x__20 => 0x__2F, 0x__60 => 0x__6F, 0x__A0 => 0x__AF, 0x__E0 => 0x__EF
    CS16_3 <= Address(5) and Address(4);                -- 0x__30 => 0x__3F, 0x__70 => 0x__7F, 0x__B0 => 0x__BF, 0x__F0 => 0x__FF

-- Decode single word address lines
    CS_0 <= (not Address(3)) and (not Address(2)) and (not Address(1)) and (not Address(0));
    -- CS_1 <= (not Address(3)) and (not Address(2)) and (not Address(1)) and Address(0);
    -- CS_2 <= (not Address(3)) and (not Address(2)) and Address(1) and (not Address(0));
    -- CS_3 <= (not Address(3)) and (not Address(2)) and Address(1) and Address(0);
    -- CS_4 <= (not Address(3)) and Address(2) and (not Address(1)) and (not Address(0));
    -- CS_5 <= (not Address(3)) and Address(2) and (not Address(1)) and Address(0);
    -- CS_6 <= (not Address(3)) and Address(2) and Address(1) and (not Address(0));
    -- CS_7 <= (not Address(3)) and Address(2) and Address(1) and Address(0);
    -- CS_8 <= Address(3) and (not Address(2)) and (not Address(1)) and (not Address(0));
    -- CS_9 <= Address(3) and (not Address(2)) and (not Address(1)) and Address(0);
    -- CS_A <= Address(3) and (not Address(2)) and Address(1) and (not Address(0));
    -- CS_B <= Address(3) and (not Address(2)) and Address(1) and Address(0);
    CS_C <= Address(3) and Address(2) and (not Address(1)) and (not Address(0));
    CS_D <= Address(3) and Address(2) and (not Address(1)) and Address(0);
    CS_E <= Address(3) and Address(2) and Address(1) and (not Address(0));
    CS_F <= Address(3) and Address(2) and Address(1) and Address(0);

----------------------------------------------------------------------------------------------------
-- REGISTER ADDRESS DECODE LOGIC. Use the decoded address strobes to define a particular address.
----------------------------------------------------------------------------------------------------

   CodeId_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_F;
   
   ModuleId_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_E;
   
   DummyCtrlModInStatus_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_D;
   
   DummyCtrlModOutStatus_cs <= CS16K_3 and CS4K_3 and CS1K_3 and CS256_3 and CS64_3 and CS16_3 and CS_C;  
     
   StartExp_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_1 and CS64_0 and CS16_0 and CS_0;

   StatOut_cs <= CS16K_0 and CS4K_0 and CS1K_0 and CS256_0 and CS64_0 and CS16_0 and CS_0;
   
----------------------------------------------------------------------------------------------------
-- REGISTER DATA LATCHING LOGIC. Use the decoded address CS strobes to enable a register write.
----------------------------------------------------------------------------------------------------
   LcbCtrlRegLatchProc : process (WbSysClk)
      begin
         if (rising_edge(WbSysClk)) then
            err_ack <= '0';						-- Start with optimism
            wrt_ack <= '0';                 -- Cancel any previous successful register write

            if (WbReset = '1') then
               StatOut_reg  <= (others => '0');
            elsif (WbWriteEnable = '1') then
               if (StatOut_cs = '1') then
                  StatOut_reg <= Data_in;
               end if;

   -- Set the error flag if the the address given does not hit any of our assigned addresses.
               if (AddrErr = '1') then                  
                  err_ack <= '1';                          		-- Signal the bus master there was an error
               else
                  wrt_ack <= '1';                          		-- Else signal for a successful register write
               end if;
            end if;
         end if;
      end process;

      
--------------------------------------------------------------------------------------------------
-- Addressing error catcher
--------------------------------------------------------------------------------------------------
   AddrErr <= not (StatOut_cs);
	

-- Catch the start exposure vector   
   StarExpProc : process(WbSysClk)
   begin
      if (rising_edge(WbSysClk) and StartExpStb_i = '1') then
         StartExp_reg <= StartExpVector_i;
      end if;
   end process;
   
----------------------------------------------------------------------------------------------------
-- REGISTER READ LOGIC. Use the **_CS strobes to multiplex the registers (or module input ports)
-- to the data_out port.
----------------------------------------------------------------------------------------------------

-- Add future registers here as they are generated.
-- Although the hierarchy here has an inherent priority, there should only be one address CS strobe
-- true on any given cycle, therefore there is no priority as to where the registers should go in this list.
	
   WbDataOut <=("0000000000000000" & StartExp_reg) 	      when (StartExp_cs = '1') else
               (StatOut_reg) 	                              when (StatOut_cs = '1') else
               (WbStatIn)              							when (DummyCtrlModInStatus_cs = '1') else
               (LocalStatus_reg)              					when (DummyCtrlModOutStatus_cs = '1') else
               (x"0000" & LCB_CONTROL_MODULE_IDENT)			when (ModuleId_cs = '1') else
               (x"0000" & CODE_IDENT)              			when (CodeId_cs = '1') else
               x"42424242"; --- Code word for bad bus read address :-)

end Behavioral;

